@@ -15,7 +15,6 @@ test_that("FeatureImp works for single output", {
   expect_s3_class(p, c("gg", "ggplot"))
   p
   
-
   var.imp = FeatureImp$new(predictor1,  loss = "mse", method = "cartesian")
   dat = var.imp$results
   # Making sure the result is sorted by decreasing importance
@@ -29,13 +28,15 @@ test_that("FeatureImp works for single output", {
   
   X.exact = data.frame(x1 = c(1,2,3), x2 = c(9,4,2))
   y.exact = c(2,3,4)
-  f.exact = Predictor$new(predict.fun = function(newdata) newdata[[1]], data = X.exact, y = y.exact)
+  f.exact = Predictor$new(predict.fun = function(newdata) newdata[["x1"]], data = X.exact, y = y.exact)
   # creates a problem on win builder
   # model.error = Metrics::mse(y.exact, f.exact$predict(X.exact))
   model.error = 1
   cart.indices = c(1, 1, 1, 2, 2, 2, 3, 3, 3)
   cartesian.error = Metrics::mse(y.exact[cart.indices], c(1, 2, 3, 1, 2, 3, 1, 2, 3))
   
+  # TODO: Check where the error comes from. Maybe something with that the predictor does not give correct results
+  # n.repetitions should be ignored
   var.imp = FeatureImp$new(f.exact, loss = "mse", method = "cartesian")
   dat = var.imp$results
   expect_class(dat, "data.frame")
@@ -54,7 +55,7 @@ test_that("FeatureImp works for single output", {
   p = var.imp$plot()
   expect_s3_class(p, c("gg", "ggplot"))
   p
-  
+
 })
 
 test_that("FeatureImp works for single output and function as loss", {
@@ -87,4 +88,12 @@ test_that("FeatureImp works for multiple output",{
 test_that("FeatureImp fails without target vector",{
   predictor2 = Predictor$new(f, data = X, predict.fun = predict.fun)
   expect_error(FeatureImp$new(predictor2, loss = "ce"))
-})
\ No newline at end of file
+})
+
+test_that("Works for different repetitions.",{
+  var.imp = FeatureImp$new(predictor1, loss = "mse", n.repetitions = 2)
+  dat = var.imp$results
+  expect_class(dat, "data.frame")
+})
+
+
